package org.openxma.dsl.platform.config;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;

import at.spardat.properties.PropertyData;
import at.spardat.properties.PropertyFileNotFoundException;
/*
 * A property resource configurer that return a merged properties instance containing both the
 * properties set on this FactoryBean and properties loaded from {@link PropertyData PropertyData}.
 */
public class XPropertiesPlaceholderConfigurer extends
        PropertyPlaceholderConfigurer {
	/**
	 * @see org.springframework.core.io.support.PropertiesLoaderSupport#mergeProperties()
	 */
    @SuppressWarnings("unchecked")
	protected Properties mergeProperties() throws IOException {
        Properties properties = super.mergeProperties();
        PropertyData propertyData = getPropertyData();
        if (propertyData!=null) {
        	for (Iterator i = propertyData.getPropertiesMap().entrySet().iterator(); i.hasNext();) {
                Map.Entry entry = (Map.Entry) i.next();
                if (entry.getValue() instanceof String) {
                    properties.put(entry.getKey(), entry.getValue());
                }
            }
        }
        return properties;
    }

	protected PropertyData getPropertyData() {
		try {
			return PropertyData.getInstance();
		} catch (PropertyFileNotFoundException propertyFileNotFoundException) {
			if (logger.isWarnEnabled()) {
				logger.warn("Could not load XProperties from : " + propertyFileNotFoundException.getMessage());
			}
		}
		return null;
	}
}
