/*******************************************************************************
 * Copyright (c) 2010 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.dsl.platform.query;

import org.openxma.dsl.platform.query.impl.ConstantExpression;
import org.openxma.dsl.platform.query.impl.LogicalExpression;
import org.openxma.dsl.platform.query.impl.RelationalExpression;
import org.openxma.dsl.platform.query.impl.SortOrderExpression;

public enum ExpressionOperator {

    ASC {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new SortOrderExpression(expression, this);
        }

    },
    DESC {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new SortOrderExpression(expression, this);
        }

    },
    LIKE {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    }, IS_NULL {

        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new LogicalExpression(expression, this, toExpression(expression, arguments));
        }

    }, IS_NOT_NULL {

        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new LogicalExpression(expression, this, toExpression(expression, arguments));
        }

    }, EQUALS {

        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    },
    GREATER_THAN {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    },

    GREATER_THAN_OR_EQUAL {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    },

    LESS_THAN {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    },

    LESS_THAN_OR_EQUAL {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }

    },

    AND {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new LogicalExpression(expression, this, toExpression(expression, arguments));
        }

    },

    OR {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new LogicalExpression(expression, this, toExpression(expression, arguments));
        }

    },
    NOT_EQUALS {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }
    },
    NOT_LIKE {
        @Override
        public Expression with(Expression expression, Object... arguments) {
            return new RelationalExpression(expression, this, toExpression(expression, arguments));
        }
    };

    public abstract Expression with(Expression expression, Object... arguments);

    private static Expression toExpression(Expression expression, Object... arguments) {
        if (isSingleArgument(arguments)) {
            return toExpression(expression, arguments[0]);
        }
        throw new IllegalStateException("multiple arguments not supported!");
    }

    private static Expression toExpression(Expression expression, Object argument) {
        if (argument instanceof Expression) {
            return (Expression) argument;
        }
        return new ConstantExpression(expression, argument);
    }

    private static boolean isSingleArgument(Object[] arguments) {
        return null != arguments && arguments.length == 1;
    }

}
