/*
 * @(#) $Id: BaseType.java 3549 2009-05-29 08:28:29Z michael_clay $
 *
 * Copyright 2004/2005 by SPARDAT Sparkassen-Datendienst Ges.m.b.H.,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.platform.valueobject;

import java.io.Serializable;

public abstract class ValueObject<T> implements Serializable {

    private static final long serialVersionUID = 1L;

    private final T value;

    public ValueObject(T value) {
        if (value == null) {
            throw new IllegalArgumentException("value must not be null in " + getClass().getSimpleName());
        }
        this.value = value;
    }

    public T getValue() {
        return value;
    }

    @Override
    public boolean equals(Object pObj) {
        if (pObj == null) {
            return false;
        }
        if (getClass() != pObj.getClass()) {
            return false;
        }
        return getValue().equals(((ValueObject<?>) pObj).getValue());
    }

    @Override
    public int hashCode() {
        return getValue().hashCode();
    }

    @Override
    public String toString() {
        return getValue().toString();
    }

    public static<V, T extends ValueObject<V>>  V valueOrNull(T valueObject) {
    	if(valueObject != null) {
    		return valueObject.getValue();
    	}
    	return null;
    }

}
