package org.openxma.dsl.platform.xma.client.browse;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

import at.spardat.xma.mdl.IWModelClient;
import at.spardat.xma.mdl.UIDelegateClient;
import at.spardat.xma.mdl.WModel;


public class BrowseMode {

    public static final byte DEFAULT = 0;
    public static final byte FOREGROUND = 1;
    public static final byte BACKGROUND = 2;
    public static final byte FONT = 4;
    public static final byte TEXT = 16;

    private static BrowseMode instance;

    public static BrowseMode getInstance() {
        if (instance == null) {
            instance = new BrowseMode();
        }
        return instance;
    }

    public Control createBrowseWidget(Control editControl, int style) {
        Control browseControl = null;
        if ((style & TEXT) != 0) {
            Text text = new Text(editControl.getParent(), SWT.LEFT | SWT.BORDER);
            text.setEditable(false);
            browseControl = text;
        } else {
            browseControl = new Label(editControl.getParent(), SWT.LEFT | SWT.WRAP);
        }
    	if ((style & BACKGROUND) != 0) {
    		browseControl.setBackground(editControl.getBackground());        	
    	}
    	if ((style & FOREGROUND) != 0) {        	
    		browseControl.setForeground(editControl.getForeground());
    	}
    	if ((style & FONT) != 0) {
    		browseControl.setFont(editControl.getFont());
    	}        
        FormData parentData = (FormData)editControl.getLayoutData();
        FormData data = cloneFormData(parentData);
        browseControl.setLayoutData(data);
        return browseControl;
    }

    public void attachBrowseWidget(IWModelClient wModel, Control browseControl) {
        UIDelegateClient uiDelegate = wModel.getUIDelegate();
        if (uiDelegate instanceof BrowseableUIDelegateClient) {
            ((BrowseableUIDelegateClient)uiDelegate).attachBrowseControl(browseControl);
        } else {
            throw new IllegalArgumentException("The ui-delegete of the widget model has to be of instance 'BrowseUIDelegateClient'");
        }
    }

    protected FormData cloneFormData(FormData parentData) {
        FormData data = new FormData();
        data.left = parentData.left;
        data.top = parentData.top;
        data.bottom = parentData.bottom;
        data.right = parentData.right;
        data.width = parentData.width;
        data.height = parentData.height;
        return data;
    }

    public void detachBrowseControls(WModel[] wModels) {
        for (int i=wModels.length-1; i>=0; i--) {
            if (wModels[i] instanceof BrowseableWMClient) {
                UIDelegateClient uIDelegateClient = ((IWModelClient)wModels[i]).getUIDelegate();
                if (uIDelegateClient instanceof BrowseableUIDelegateClient) {
                    ((BrowseableUIDelegateClient)uIDelegateClient).detachBrowseControl();
                }
            }
        }
    }

    static public void updateBrowseControl(Control browseControl, String text) {
        if (text != null) {
            if (browseControl instanceof Label) {
                ((Label)browseControl).setText(escapeLabelText(text));
            } else if (browseControl instanceof Text) {
                ((Text)browseControl).setText(text);
            }
        }
    }
    
    public static String escapeLabelText(String text) {
        if (text == null || text.length() == 0 || text.indexOf('&') < 0) {
            return text;
        }
        return text.replaceAll("&", "&&");
    }

    public static String unEscapeLabelText(String text) {
        if (text == null || text.length() == 0 || text.indexOf("&&") < 0) {
            return text;
        }
        return text.replaceAll("&&", "&");
    }
    
}
